@@ -4,6 +4,7 @@ import aiohttp
 import asyncio
 import ujson as json
 from uuid import UUID
+from typing import Union
 from .sockets import Wss
 from typing import BinaryIO
 from binascii import hexlify
@@ -17,8 +18,9 @@ class SClient(Wss):
     def __init__(self, deviceId: str = None, Trace: bool = False):
         self.Trace = Trace
         self.uid = None
+        self.secret = None
+        
         headers.deviceId = deviceId
-
         self.deviceId = headers.Headers().deviceId
         self.headers = headers.Headers().headers
         self.web_headers = headers.Headers().web_headers
@@ -59,26 +61,38 @@ class SClient(Wss):
             if req.status != 200:return CheckExceptions(await req.json())
             else:return info
 
-    async def login(self, email: str, password: str, socket: bool = False):
-        data = json.dumps({
-            "email": email,
-            "secret": f"0 {password}",
+    async def login(self, email: str = None, password: str = None, secret: str = None,socket: bool = False):
+        data = {
             "clientType": 100,
             "action": "normal",
             "deviceID": self.deviceId,
             "v": 2,
             "timestamp": int(timestamp() * 1000)
-        })
+        }
+
+        ## - email n pass or secret login hanlder
+        if password:
+            if email:
+                data["email"] = email
+                data["secret"] = f"0 {password}"
+            else: raise ValueError("When using password email is required when using secret email is not required so please insert email with password")
+        elif secret: data["secret"] = secret
+        else: raise TypeError("Please consider inserting email and password or secret")
+
+        data = json.dumps(data)
         async with self.session.post(api(f"/g/s/auth/login"), headers=headers.Headers(data=data).headers, data=data) as req:
             if req.status != 200:
                 return CheckExceptions(await req.json())
             else:
-                self.sid = f"sid={(await req.json())['sid']}"
+                self.sid = f'sid={(await req.json())["sid"]}'
                 self.uid = (await req.json())["auid"]
                 self.headers["NDCAUTH"] = self.sid
+                self.secret = (await req.json())["secret"]
                 headers.sid = self.sid
                 headers.uid = self.uid
-                if socket:await self.Start()
+                self.headers = headers.Headers().headers
+                self.web_headers = headers.Headers().web_headers
+                if socket: self.Launch()
                 else: pass
                 return Login(await req.json())
 
@@ -211,7 +225,7 @@ class SClient(Wss):
             if req.status != 200: return CheckExceptions(((await req.json())))
             return Json(((await req.json())))
 
-    async def start_chat(self, userId: [str, list], title: str = None, message: str = None, content: str = None,chatType: int = 0):
+    async def start_chat(self, userId: Union[str, list], title: str = None, message: str = None, content: str = None,chatType: int = 0):
         if type(userId) is list:
             userIds = userId
         elif type(userId) is str:
@@ -288,12 +302,41 @@ class SClient(Wss):
             if req.status != 200: return CheckExceptions(await req.json())
             return Json((await req.json()))
 
+    async def flag(self, reason: str, type: str = "spam", userId: str = None, wikiId: str = None, blogId: str = None):
+        types = {"violence": 106, "hate": 107, "suicide": 108, "troll": 109, "nudity": 110, "bully": 0, "off-topic": 4, "spam": 2}
+
+        data = {
+            "message": reason,
+            "timestamp": int(timestamp() * 1000)
+        }
+
+        if type in types:
+            data["flagType"] = types[type]
+
+        if userId:
+            data["objectId"] = userId
+            data['objectType'] = 0
+
+        elif blogId:
+            data["objectId"] = blogId
+            data['objectType'] = 1
+
+        elif wikiId:
+            data["objectId"] = wikiId
+            data["objectType"] = 2
+        else: raise TypeError("choose a certain type to report")
+
+        data = json.dumps(data)
+        async with self.session.post(api(f"/g/s/flag", headers=headers.Headers(data=data).headers, data=data)) as req:
+            if req.status != 200: return CheckExceptions(await req.json())
+            else: return Json(await req.json())
+
     async def unfollow(self, userId: str):
         async with self.session.post(api(f"/g/s/user-profile/{userId}/member/{self.uid}"), headers=self.headers ) as req:
             if req.status != 200: return CheckExceptions(await req.json())
             return Json((await req.json()))
 
-    async def follow(self, userId: [str, list]):
+    async def follow(self, userId: Union[str, list]):
         if type(userId) is str:
             async with self.session.post(api(f"/g/s/user-profile/{userId}/member"), headers=self.headers ) as req:
                 if req.status != 200: return CheckExceptions(await req.json())
@@ -392,6 +435,7 @@ class SClient(Wss):
             data["type"] = 3
 
         if snippetLink and snippetImage:
+            data["attachedObject"] = None
             data["extensions"]["linkSnippetList"] = [{
                 "link": snippetLink,
                 "mediaType": 100,
@@ -428,7 +472,7 @@ class SClient(Wss):
             data["extensions"] = None
 
         data = json.dumps(data)
-        async with self.session.post(api(f"/g/s/chat/thread/{chatId}/message/{message}"),headers=headers.Headers(data=data).headers , data=data) as req:
+        async with self.session.post(api(f"/g/s/chat/thread/{chatId}/message"),headers=headers.Headers(data=data).headers , data=data) as req:
             if req.status != 200: return CheckExceptions(await req.json())
             return Json((await req.json()))
 
@@ -546,7 +590,7 @@ class SClient(Wss):
             if req.status != 200: return CheckExceptions(await req.json())
             return Json((await req.json()))
 
-    async def invite_by_host(self, chatId: str, userId: [str, list]):
+    async def invite_by_host(self, chatId: str, userId: Union[str, list]):
         data = json.dumps({"uidList": userId, "timestamp": int(timestamp() * 1000)})
 
         async with self.session.post(api(f"/g/s/chat/thread/{chatId}/avchat-members"),headers=headers.Headers(data=data).headers , data=data) as req:
