@@ -4,6 +4,7 @@ import aiohttp
 import asyncio
 import ujson as json
 from uuid import UUID
+from typing import Union
 from time import timezone
 from typing import BinaryIO
 from binascii import hexlify
@@ -77,7 +78,7 @@ class SLocal:
             else: return UserProfileList((await req.json())["userProfileList"]).UserProfileList
 
     async def get_member_followers(self, userId: str = None, start: int = 0, size: int = 25):
-        async with self.session.get(api(f"/x{self.comId}/s/user-profile/{userId}/joined?start={start}&size={size}", headers = self.headers)) as req:
+        async with self.session.get(api(f"/x{self.comId}/s/user-profile/{userId}/member?start={start}&size={size}"), headers = self.headers) as req:
             if req.status != 200: return CheckExceptions(await req.json())
             else: return UserProfileList((await req.json())["userProfileList"]).UserProfileList
 
@@ -233,7 +234,7 @@ class SLocal:
             if req.status != 200: return CheckExceptions(await req.json())
             return Json(await req.json())
 
-    async def follow(self, userId: [str, list]):
+    async def follow(self, userId: Union[str, list]):
         data = {"timestamp": int(timestamp() * 1000)}
 
         if isinstance(userId, str): url = api(f"/x{self.comId}/s/user-profile/{userId}/member")
@@ -247,7 +248,7 @@ class SLocal:
             if req.status != 200: return CheckExceptions(await req.json())
             return Json(await req.json())
 
-    async def start_chat(self, userId: [str, list], title: str = None, message: str = None, content: str = None, type: int = 0):
+    async def start_chat(self, userId: Union[str, list], title: str = None, message: str = None, content: str = None, type: int = 0):
         if isinstance(userId, list): userIds = userId
         elif isinstance(userId, str): userIds = [userId]
         else: raise TypeError("Please put a str or list of userId")
@@ -265,7 +266,7 @@ class SLocal:
             if req.status != 200: return CheckExceptions(await req.json())
             return Json(await req.json())
 
-    async def invite_to_chat(self, userId: [str, list], chatId: str = None):
+    async def invite_to_chat(self, userId: Union[str, list], chatId: str = None):
         if isinstance(userId, str): userIds = [userId]
         elif isinstance(userId, list): userIds = userId
         else: raise TypeError("")
@@ -319,7 +320,7 @@ class SLocal:
             res.append(Json(await req.json()))
             return res
 
-    async def chat_settings(self, chatId: str, viewOnly: bool = False, doNotDisturb: bool = True, canInvite: bool = False, canTip: bool = None, pin: bool = None, coHosts: [str, list] = None):
+    async def chat_settings(self, chatId: str, viewOnly: bool = False, doNotDisturb: bool = True, canInvite: bool = False, canTip: bool = None, pin: bool = None, coHosts: Union[str, list] = None):
         res = []
 
         if doNotDisturb:
@@ -400,13 +401,20 @@ class SLocal:
             return Json(await req.json())
 
     async def like_comment(self, commentId: str, blogId: str = None, wikiId: str = None, userId: str = None):
-        data = json.dumps({"value": 1})
-        if blogId: url = api(f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}/vote?cv=1.2&value=1")
-        elif wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}/comment/{commentId}/vote?cv=1.2&value=1")
-        elif userId: url = api(f"/x{self.comId}/s/user-profile/{userId}/comment/{commentId}/vote?cv=1.2&value=1")
+        data = {"value": 1}
+        if blogId:
+            data["eventSource"] = "PostDetailView"
+            url = api(f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}/vote?cv=1.2&value=1")
+        elif wikiId:
+            data["eventSource"] = "PostDetailView"
+            url = api(f"/x{self.comId}/s/item/{wikiId}/comment/{commentId}/vote?cv=1.2&value=1")
+        elif userId:
+            data["eventSource"] = "UserProfileView"
+            url = api(f"/x{self.comId}/s/user-profile/{userId}/comment/{commentId}/vote?cv=1.2&value=1")
         else: raise TypeError("Please put a wiki or user or blog Id")
 
-        async with self.session.post(url, data=data, headers=headers) as req:
+        data = json.dumps(data)
+        async with self.session.post(url, data=data, headers=headers.Headers(data=data).headers) as req:
             if req.status != 200: return CheckExceptions(await req.json())
             return Json(await req.json())
 
@@ -416,7 +424,7 @@ class SLocal:
         elif userId: url = api(f"/x{self.comId}/s/user-profile/{userId}/comment/{commentId}/g-vote?eventSource=UserProfileView")
         else: raise TypeError("Please put a wiki or user or blog Id")
 
-        async with self.session.delete(url, headers=headers) as req:
+        async with self.session.delete(url, headers=self.headers) as req:
             if req.status != 200: return CheckExceptions(await req.json())
             return Json(await req.json())
 
@@ -574,8 +582,8 @@ class SLocal:
                 if req.status != 200: return CheckExceptions(await req.json())
                 else: return Json(await req.json())
 
-    async def invite_by_host(self, chatId: str, userId: [str, list]):
-        async with self.session.post(api(f"/x{self.comId}/s/chat/thread/{chatId}/member/{userId}/invite-av-chat"), headers=headers.Headers(data=data).headers) as req:
+    async def invite_by_host(self, chatId: str, userId: Union[str, list]):
+        async with self.session.post(api(f"/x{self.comId}/s/chat/thread/{chatId}/member/{userId}/invite-av-chat"), headers=self.headers) as req:
             if req.status != 200: return CheckExceptions(await req.json())
             else: return Json(await req.json())
 
@@ -699,7 +707,7 @@ class SLocal:
             if req.status != 200: return CheckExceptions(await req.json())
             return Json(await req.json())
 
-    async def post_wiki(self, title: str, content: str, fansOnly: bool = False, icon: str = None, backgroundColor: str = None, keywords: [str, list] = None):
+    async def post_wiki(self, title: str, content: str, fansOnly: bool = False, icon: str = None, backgroundColor: str = None, keywords: Union[str, list] = None):
         data = {
             "extensions": {"fansOnly": fansOnly, "props": [], "style": {"backgroundColor": backgroundColor}},
             "content": content,
@@ -829,6 +837,35 @@ class SLocal:
             if "OK" not in (await req.json())["api:message"]: return CheckExceptions(await req.json())
             return Json(await req.json())
 
+    async def flag(self, reason: str, type: str = "spam", userId: str = None, wikiId: str = None, blogId: str = None):
+        types = {"violence": 106, "hate": 107, "suicide": 108, "troll": 109, "nudity": 110, "bully": 0, "off-topic": 4, "spam": 2}
+
+        data = {
+            "message": reason,
+            "timestamp": int(timestamp() * 1000)
+        }
+
+        if type in types:
+            data["flagType"] = types[type]
+
+        if userId:
+            data["objectId"] = userId
+            data['objectType'] = 0
+
+        elif blogId:
+            data["objectId"] = blogId
+            data['objectType'] = 1
+
+        elif wikiId:
+            data["objectId"] = wikiId
+            data["objectType"] = 2
+        else: raise TypeError("choose a certain type to report")
+
+        data = json.dumps(data)
+        async with self.session.post(api(f"/x{self.comId}/s/flag", headers=headers.Headers(data=data).headers, data=data)) as req:
+            if req.status != 200: return CheckExceptions(await req.json())
+            else: return Json(await req.json())
+
     async def send_active_time(self,tz: int = int(-timezone // 1000), timers: list = None):
         data = {
             "userActiveTimeChunkList": [{"start": int(timestamp()), "end": int(timestamp() + 300)}],
@@ -895,7 +932,7 @@ class SLocal:
 
     async def get_quiz_rankings(self, quizId: str, start: int = 0, size: int = 25):
         async with self.session.get(api(f"/x{self.comId}/s/blog/{quizId}/quiz/result?start={start}&size={size}"),headers=self.headers) as req:
-            if req.status != 200:return CheckException(await req.json())
+            if req.status != 200:return CheckExceptions(await req.json())
             else:return QuizRankings(await req.json()).QuizRankings
 
     async def search_user(self, username: str, start: int = 0, size: int = 25):
