@@ -7,6 +7,7 @@ import threading
 import websockets
 from ..lib import *
 import ujson as json
+from typing import Union
 from typing import BinaryIO
 from sys import _getframe as getframe
 from ..lib.objects import Event, Payload
@@ -286,7 +287,7 @@ class Actions:
         self.default()
         return SetAction(self.socket, data)
 
-    def Custom(self, actions: [str, list], target: str, params: dict):
+    def Custom(self, actions: Union[str, list], target: str, params: dict):
 
         data = {
             "o": {
