@@ -2,6 +2,7 @@ import os
 import requests
 import ujson as json
 from uuid import UUID
+from typing import Union
 from typing import BinaryIO
 from binascii import hexlify
 from time import time as timestamp
@@ -17,7 +18,7 @@ class Client(Wss):
         self.trace = trace
         self.proxies = proxies
         self.uid = None
-
+        self.secret = None
         headers.deviceId = deviceId
         Wss.__init__(self, self, trace=self.trace)
         self.deviceId = headers.Headers().deviceId
@@ -44,17 +45,25 @@ class Client(Wss):
         if req.status_code != 200: return CheckExceptions(req.json())
         else: return info
 
-    def login(self, email: str, password: str, socket: bool = False):
-        data = json.dumps({
-            "email": email,
-            "secret": f"0 {password}",
+    def login(self, email: str = None, password: str = None, secret: str = None,socket: bool = False):
+        data = {
             "clientType": 100,
             "action": "normal",
             "deviceID": self.deviceId,
             "v": 2,
             "timestamp": int(timestamp() * 1000)
-        })
+        }
 
+        ## - email n pass or secret login hanlder
+        if password:
+            if email:
+                data["email"] = email
+                data["secret"] = f"0 {password}"
+            else: raise ValueError("When using password email is required when using secret email is not required so please insert email with password")
+        elif secret: data["secret"] = secret
+        else: raise TypeError("Please consider inserting email and password or secret")
+
+        data = json.dumps(data)
         req = self.session.post(api(f"/g/s/auth/login"), headers=headers.Headers(data=data).headers, proxies=self.proxies,data=data)
         if req.status_code != 200:
             return CheckExceptions(req.json())
@@ -62,6 +71,7 @@ class Client(Wss):
             self.sid = f'sid={req.json()["sid"]}'
             self.uid = req.json()["auid"]
             self.headers["NDCAUTH"] = self.sid
+            self.secret = req.json()["secret"]
             headers.sid = self.sid
             headers.uid = self.uid
             self.headers = headers.Headers().headers
@@ -210,7 +220,7 @@ class Client(Wss):
         if req.status_code != 200: return CheckExceptions(req.json())
         return Json(req.json())
 
-    def start_chat(self, userId: [str, list], title: str = None, message: str = None, content: str = None,
+    def start_chat(self, userId: Union[str, list], title: str = None, message: str = None, content: str = None,
                    chatType: int = 0):
         if type(userId) is list:
             userIds = userId
@@ -287,9 +297,38 @@ class Client(Wss):
         data = {"timestamp": int(timestamp() * 1000)}
         if InviteId: data["invitationId"] = InviteId
         data = json.dumps(data)
-        req = self.session.post(api(f"/x{comId}/s/community/join?{self.sid}"), data=data,headers=headers.Headers(data=data).headers, proxies=self.proxies)
+        req = self.session.post(api(f"/x{comId}/s/community/join"), data=data,headers=headers.Headers(data=data).headers, proxies=self.proxies)
         if req.status_code != 200: return CheckExceptions(req.json())
         return Json(req.json())
+        
+    def flag(self, reason: str, type: str = "spam", userId: str = None, wikiId: str = None, blogId: str = None):
+        types = {"violence": 106, "hate": 107, "suicide": 108, "troll": 109, "nudity": 110, "bully": 0, "off-topic": 4, "spam": 2}
+
+        data = {
+            "message": reason,
+            "timestamp": int(timestamp() * 1000)
+        }
+
+        if type in types:
+            data["flagType"] = types[type]
+
+        if userId:
+            data["objectId"] = userId
+            data['objectType'] = 0
+
+        elif blogId:
+            data["objectId"] = blogId
+            data['objectType'] = 1
+
+        elif wikiId:
+            data["objectId"] = wikiId
+            data["objectType"] = 2
+        else: raise TypeError("choose a certain type to report")
+
+        data = json.dumps(data)
+        req = self.session.post(api(f"/g/s/flag", headers=headers.Headers(data=data).headers, proxies=self.proxies, data=data))
+        if req.status_code != 200: return CheckExceptions(req.json())
+        else: return Json(req.json())
 
     def unfollow(self, userId: str):
         req = self.session.post(api(f"/g/s/user-profile/{userId}/member/{self.uid}"), headers=self.headers,
@@ -297,7 +336,7 @@ class Client(Wss):
         if req.status_code != 200: return CheckExceptions(req.json())
         return Json(req.json())
 
-    def follow(self, userId: [str, list]):
+    def follow(self, userId: Union[str, list]):
         if type(userId) is str:
             req = self.session.post(api(f"/g/s/user-profile/{userId}/member"), headers=self.headers, proxies=self.proxies)
         elif type(userId) is list:
@@ -399,6 +438,7 @@ class Client(Wss):
             data["type"] = 3
 
         if snippetLink and snippetImage:
+            data["attachedObject"] = None
             data["extensions"]["linkSnippetList"] = [{
                 "link": snippetLink,
                 "mediaType": 100,
@@ -435,7 +475,7 @@ class Client(Wss):
             data["extensions"] = None
 
         data = json.dumps(data)
-        req = self.session.post(api(f"/g/s/chat/thread/{chatId}/message/{message}"),
+        req = self.session.post(api(f"/g/s/chat/thread/{chatId}/message"),
                             headers=headers.Headers(data=data).headers, proxies=self.proxies, data=data)
         if req.status_code != 200: return CheckExceptions(req.json())
         return Json(req.json())
@@ -559,7 +599,7 @@ class Client(Wss):
         if req.status_code != 200: return CheckExceptions(req.json())
         return Json(req.json())
 
-    def invite_by_host(self, chatId: str, userId: [str, list]):
+    def invite_by_host(self, chatId: str, userId: Union[str, list]):
         data = json.dumps({"uidList": userId, "timestamp": int(timestamp() * 1000)})
 
         req = self.session.post(api(f"/g/s/chat/thread/{chatId}/avchat-members"),
