@@ -1,7 +1,9 @@
+from email import header
 import os
 import requests
 import ujson as json
 from uuid import UUID
+from typing import Union
 from time import timezone
 from typing import BinaryIO
 from binascii import hexlify
@@ -43,11 +45,10 @@ class Local:
         else: raise TypeError("")
 
         data = file.read()
-        header = self.headers
-        header["content-type"] = type
-        header["content-length"] = str(len(data))
+        self.headers["content-type"] = type
+        self.headers["content-length"] = str(len(data))
 
-        req = self.session.post(api("/g/s/media/upload"), data=data, headers=header)
+        req = self.session.post(api("/g/s/media/upload"), data=data, headers=self.headers, proxies=self.proxies)
         if req.status_code != 200: return CheckExceptions(req.json())
         else: return req.json()["mediaValue"]
 
@@ -62,7 +63,7 @@ class Local:
         else: return UserProfileList(req.json()["userProfileList"]).UserProfileList
 
     def get_member_followers(self, userId: str = None, start: int = 0, size: int = 25):
-        req = self.session.get(api(f"/x{self.comId}/s/user-profile/{userId}/joined?start={start}&size={size}"))
+        req = self.session.get(api(f"/x{self.comId}/s/user-profile/{userId}/member?start={start}&size={size}"), headers=self.headers, proxies=self.proxies)
         if req.status_code != 200: return CheckExceptions(req.json())
         else: return UserProfileList(req.json()["userProfileList"]).UserProfileList
 
@@ -152,7 +153,7 @@ class Local:
                 "mediaUploadValue": base64.b64encode(snippetImage.read()).decode(),
                 "mediaUploadValueContentType": "image/png"
             }]
-
+            
         if ytVideo:
             data["content"] = None
             data["mediaType"] = 103
@@ -213,7 +214,7 @@ class Local:
         if req.status_code != 200: return CheckExceptions(req.json())
         return Json(req.json())
 
-    def follow(self, userId: [str, list]):
+    def follow(self, userId: Union[str, list]):
         data = {"timestamp": int(timestamp() * 1000)}
 
         if isinstance(userId, str): url = api(f"/x{self.comId}/s/user-profile/{userId}/member")
@@ -227,7 +228,7 @@ class Local:
         if req.status_code != 200: return CheckExceptions(req.json())
         return Json(req.json())
 
-    def start_chat(self, userId: [str, list], title: str = None, message: str = None, content: str = None, type: int = 0):
+    def start_chat(self, userId: Union[str, list], title: str = None, message: str = None, content: str = None, type: int = 0):
         if isinstance(userId, list): userIds = userId
         elif isinstance(userId, str): userIds = [userId]
         else: raise TypeError("Please put a str or list of userId")
@@ -245,7 +246,7 @@ class Local:
         if req.status_code != 200: return CheckExceptions(req.json())
         return Json(req.json())
 
-    def invite_to_chat(self, userId: [str, list], chatId: str = None):
+    def invite_to_chat(self, userId: Union[str, list], chatId: str = None):
         if isinstance(userId, str): userIds = [userId]
         elif isinstance(userId, list): userIds = userId
         else: raise TypeError("")
@@ -299,7 +300,7 @@ class Local:
         res.append(Json(req.json()))
         return res
 
-    def chat_settings(self, chatId: str, viewOnly: bool = False, doNotDisturb: bool = True, canInvite: bool = False, canTip: bool = None, pin: bool = None, coHosts: [str, list] = None):
+    def chat_settings(self, chatId: str, viewOnly: bool = False, doNotDisturb: bool = True, canInvite: bool = False, canTip: bool = None, pin: bool = None, coHosts: Union[str, list] = None):
         res = []
 
         if doNotDisturb:
@@ -380,13 +381,20 @@ class Local:
         return Json(req.json())
 
     def like_comment(self, commentId: str, blogId: str = None, wikiId: str = None, userId: str = None):
-        data = json.dumps({"value": 1})
-        if blogId: url = api(f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}/vote?cv=1.2&value=1")
-        elif wikiId: url = api(f"/x{self.comId}/s/item/{wikiId}/comment/{commentId}/vote?cv=1.2&value=1")
-        elif userId: url = api(f"/x{self.comId}/s/user-profile/{userId}/comment/{commentId}/vote?cv=1.2&value=1")
+        data = {"value": 1}
+        if blogId:
+            data["eventSource"] = "PostDetailView"
+            url = api(f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}/vote?cv=1.2&value=1")
+        elif wikiId:
+            data["eventSource"] = "PostDetailView"
+            url = api(f"/x{self.comId}/s/item/{wikiId}/comment/{commentId}/vote?cv=1.2&value=1")
+        elif userId:
+            data["eventSource"] = "UserProfileView"
+            url = api(f"/x{self.comId}/s/user-profile/{userId}/comment/{commentId}/vote?cv=1.2&value=1")
         else: raise TypeError("Please put a wiki or user or blog Id")
 
-        req = self.session.post(url, data=data, headers=headers)
+        data = json.dumps(data)
+        req = self.session.post(url, data=data, headers=headers.Headers(data=data).headers)
         if req.status_code != 200: return CheckExceptions(req.json())
         return Json(req.json())
 
@@ -396,7 +404,7 @@ class Local:
         elif userId: url = api(f"/x{self.comId}/s/user-profile/{userId}/comment/{commentId}/g-vote?eventSource=UserProfileView")
         else: raise TypeError("Please put a wiki or user or blog Id")
 
-        req = self.session.delete(url, headers=headers)
+        req = self.session.delete(url, headers=self.headers)
         if req.status_code != 200: return CheckExceptions(req.json())
         return Json(req.json())
 
@@ -549,7 +557,7 @@ class Local:
         if req.status_code != 200: return CheckExceptions(req.json())
         else: return Json(req.json())
 
-    def invite_by_host(self, chatId: str, userId: [str, list]):
+    def invite_by_host(self, chatId: str, userId: Union[str, list]):
         req = self.session.post(api(f"/x{self.comId}/s/chat/thread/{chatId}/member/{userId}/invite-av-chat"), headers=self.headers, proxies=self.proxies)
         if req.status_code != 200: return CheckExceptions(req.json())
         else: return Json(req.json())
@@ -673,7 +681,7 @@ class Local:
         if req.status_code != 200: return CheckExceptions(req.json())
         return Json(req.json())
 
-    def post_wiki(self, title: str, content: str, fansOnly: bool = False, icon: str = None, backgroundColor: str = None, keywords: [str, list] = None):
+    def post_wiki(self, title: str, content: str, fansOnly: bool = False, icon: str = None, backgroundColor: str = None, keywords: Union[str, list] = None):
         data = {
             "extensions": {"fansOnly": fansOnly, "props": [], "style": {"backgroundColor": backgroundColor}},
             "content": content,
@@ -792,6 +800,35 @@ class Local:
         if "OK" not in req.json()["api:message"]: return CheckExceptions(req.json())
         return Json(req.json())
 
+    def flag(self, reason: str, type: str = "spam", userId: str = None, wikiId: str = None, blogId: str = None):
+        types = {"violence": 106, "hate": 107, "suicide": 108, "troll": 109, "nudity": 110, "bully": 0, "off-topic": 4, "spam": 2}
+
+        data = {
+            "message": reason,
+            "timestamp": int(timestamp() * 1000)
+        }
+
+        if type in types:
+            data["flagType"] = types[type]
+
+        if userId:
+            data["objectId"] = userId
+            data['objectType'] = 0
+
+        elif blogId:
+            data["objectId"] = blogId
+            data['objectType'] = 1
+
+        elif wikiId:
+            data["objectId"] = wikiId
+            data["objectType"] = 2
+        else: raise TypeError("choose a certain type to report")
+
+        data = json.dumps(data)
+        req = self.session.post(api(f"/x{self.comId}/s/flag", headers=headers.Headers(data=data).headers, proxies=self.proxies, data=data))
+        if req.status_code != 200: return CheckExceptions(req.json())
+        else: return Json(req.json())
+
     def send_active_time(self,tz: int = int(-timezone // 1000), timers: list = None):
         data = {
             "userActiveTimeChunkList": [{"start": int(timestamp()), "end": int(timestamp() + 300)}],
@@ -858,7 +895,7 @@ class Local:
 
     def get_quiz_rankings(self, quizId: str, start: int = 0, size: int = 25):
         req = self.session.get(api(f"/x{self.comId}/s/blog/{quizId}/quiz/result?start={start}&size={size}"),headers=self.headers, proxies=self.proxies)
-        if req.status_code != 200:return CheckException(req.json())
+        if req.status_code != 200: return CheckExceptions(req.json())
         else:return QuizRankings(req.json()).QuizRankings
 
     def search_user(self, username: str, start: int = 0, size: int = 25):
