@@ -5,6 +5,7 @@ import websocket
 import threading
 import time as timer
 import ujson as json
+from typing import Union
 from typing import BinaryIO
 from sys import _getframe as getframe
 
@@ -317,7 +318,7 @@ class Actions:
         self.default()
         return SetAction(self.socket, data)
 
-    def Custom(self, actions: [str, list], target: str, params: dict):
+    def Custom(self, actions: Union[str, list], target: str, params: dict):
         """
         Send Custom Action
 
